#!/bin/bash

exit_trap() {
    local lc="$BASH_COMMAND" rc=$?
    echo "Command [$lc] exited with ERROR CODE [$rc]"
    echo ""
}

prepare_rhel_like() {
    echo "Installing needed packages rhel way"
    /bin/rpm --query --quiet epel-release || $install_command -y install epel-release
    $install_command -y install $packages
}

prepare_debian_like() {
    echo "Installing needed packages debian way"
    install_command="/usr/bin/apt-get"
    $install_command update
    $install_command --force-yes -y --fix-missing install $packages
}

prepare_suse_like() {
    echo "Installing needed packages suse way"
    install_command="/usr/bin/zypper"
    $install_command -n install $packages
}

setup_service() {
    echo "Setup plugins and service"
    /bin/mkdir -p /usr/lib/nagios-ngcs-plugins
    /bin/cp -f ${script_path}linux/plugins/* /usr/lib/nagios-ngcs-plugins/
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_mem_usage.pl
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_procs.pl
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_service_cmd
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_cpu
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_unix_disk_usage.pl
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_lsi_raid
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_md_raid
    /bin/chmod 755 /usr/lib/nagios-ngcs-plugins/check_ngcs_raid
    /bin/cp -f ${script_path}linux/config/nrpe.cfg $config_path/

    if [ -e /usr/local/sbin/storcli64 -o -e /usr/local/sbin/perccli64 ]; then
        echo "Configuring binaries"
        if [ ! -d /etc/sudoers.d ]; then
            $install_command -y install sudo
        fi
        echo "Giving perms to binaries"
        cat <<EOF >/etc/sudoers.d/check_raid
nagios ALL=(root)NOPASSWD:/usr/local/sbin/storcli64
nagios ALL=(root)NOPASSWD:/usr/local/sbin/perccli64
EOF
        /bin/chmod 440 /etc/sudoers.d/check_raid
    fi
}

setup_container() {
    useradd -m monitoring
    loginctl enable-linger monitoring
    su - monitoring -c "mkdir -p /home/monitoring/.config/systemd/user/"
    su - monitoring -c "cat << EOF_SERVICE > /home/monitoring/.config/systemd/user/monitoring.service
[Unit]
Description=Monitoring daemon
Wants=network-online.target
After=network-online.target

[Service]
Restart=on-failure
TimeoutStopSec=70
ExecStart=/usr/bin/podman run \
    --pull newer \
    --replace \
    --restart on-failure:5 \
    --detach \
    --net=host \
    --pid=host \
    --name=monitoring \
    docker.io/arsysdevelopment/nrpe:latest
ExecStop=/usr/bin/podman stop -t 10 monitoring
ExecStopPost=/usr/bin/podman stop -t 10 monitoring
Type=forking

[Install]
WantedBy=default.target
EOF_SERVICE"
    su - monitoring -c "export XDG_RUNTIME_DIR=/run/user/$(id -u monitoring) && systemctl --user enable --now monitoring.service"
}

fix_plugins() {
    echo "Fixing plugins"
    /bin/sed -i 's/Nagios::Plugin/Monitoring::Plugin/g' /usr/lib/nagios-ngcs-plugins/*.pl
}

fix_service() {
    echo "Fixing configuration"
    /bin/sed -i 's/pid_file=.*/pid_file=\/var\/run\/nrpe\/nrpe.pid/' $config_path/nrpe.cfg
    /bin/sed -i 's/nrpe_user=.*/nrpe_user=nrpe/' $config_path/nrpe.cfg
    /bin/sed -i 's/nrpe_group=.*/nrpe_group=nrpe/' $config_path/nrpe.cfg
}

enable_systemd_service() {
      echo "Enabling systemd service"
      systemctl_var=$(which systemctl)
      $systemctl_var enable "$service"
      $systemctl_var restart "$service"
}

enable_updaterc_service() {
    echo "Enabling updaterc service"
    /usr/sbin/update-rc.d "$service" defaults
    /etc/init.d/"$service" restart
}

enable_chkconfig_service() {
    echo "Enabling chkconfig service"
    /sbin/chkconfig "$service" on
    /sbin/service "$service" restart
}

trap exit_trap EXIT
set -e

script_path=${0%%$(basename $0)}
config_path="/etc/nagios"

# Parse distro

arch=$(uname -m)
source /etc/os-release

mydistro="$ID""$VERSION_ID"
echo "Working in $mydistro"

case $mydistro in
rhel7* | centos7)
    install_command="/usr/bin/yum"
    packages="bc fping perl-Crypt-DES perl-Digest-HMAC perl-Digest-SHA1 perl-Net-SNMP perl-Socket6 nagios-nrpe nagios-plugins sysstat perl-Class-Accessor perl-Config-Tiny perl-Params-Validate perl-Math-Calc-Units perl-Nagios-Plugin perl-File-Slurp perl-Readonly"
    service="nrpe.service"
    prepare_rhel_like
    setup_service
    fix_service
    enable_systemd_service
    ;;
rhel8* | centos8 | almalinux8* | rocky8*)
    install_command="/usr/bin/dnf"
    packages="bc fping perl-Crypt-DES perl-Digest-HMAC perl-Socket6 nagios-nrpe nagios-plugins sysstat perl-Config-Tiny perl-Math-Calc-Units perl-File-Slurp"
    service="nrpe.service"
    prepare_rhel_like
    $install_command -y localinstall ${script_path}linux/centos/64/{perl-Class-Accessor-0.51-2.fc28.noarch.rpm,perl-Module-Implementation-0.09-15.fc28.noarch.rpm,perl-Monitoring-Plugin-0.39-9.fc28.noarch.rpm,perl-Nagios-Plugin-0.37-9.fc28.noarch.rpm,perl-Params-Validate-1.29-4.fc28.x86_64.rpm,perl-Readonly-2.05-5.fc28.noarch.rpm}
    setup_service
    fix_service
    enable_systemd_service
    ;;
centos9 | almalinux9* | rocky9*)
    /usr/bin/dnf clean all
    /bin/rpm --query --quiet podman slirp4netns || /usr/bin/dnf -y install podman slirp4netns
    /sbin/setcap cap_setuid=ep /usr/bin/newuidmap
    /sbin/setcap cap_setgid=ep /usr/bin/newgidmap
    setup_container
    ;;
ubuntu16.04)
    packages="bc nagios-plugins libfile-slurp-perl libnagios-plugin-perl libreadonly-perl"
    service="nagios-nrpe-server-args"
    prepare_debian_like
    if [ ${arch} == "x86_64" ]; then
        dpkg -i ${script_path}linux/ubuntu/nagios-nrpe-server-args_2.15-1ubuntu2_amd64.deb
    else
        dpkg -i ${script_path}linux/ubuntu/nagios-nrpe-server-args_2.15-1ubuntu2_i386.deb
    fi
    setup_service
    enable_updaterc_service
    ;;
ubuntu18.04 | ubuntu20.04 | ubuntu22.04 | ubuntu24.04)
    packages="bc nagios-plugins libfile-slurp-perl libmonitoring-plugin-perl libreadonly-perl nagios-nrpe-server"
    service="nagios-nrpe-server"
    prepare_debian_like
    setup_service
    fix_plugins
    enable_updaterc_service
    ;;
debian9)
    packages="bc nagios-plugins libfile-slurp-perl libreadonly-perl libtest-pod-perl libclass-accessor-perl libparams-validate-perl libconfig-tiny-perl libmath-calc-units-perl libnagios-object-perl sysstat libmonitoring-plugin-perl"
    service="nagios-nrpe-server-args"
    prepare_debian_like
    if [ ${arch} == "x86_64" ]; then
        dpkg -i ${script_path}linux/debian/nagios-nrpe-server-args_3.0.1-3+deb9u1.1_amd64.deb
    else
        dpkg -i ${script_path}linux/debian/nagios-nrpe-server-args_3.0.1-3+deb9u1.1_i386.deb
    fi
    setup_service
    fix_plugins
    enable_updaterc_service
    ;;
debian10 | debian11 | debian12)
    packages="bc nagios-plugins libfile-slurp-perl libreadonly-perl libtest-pod-perl libclass-accessor-perl libparams-validate-perl libconfig-tiny-perl libmath-calc-units-perl libnagios-object-perl sysstat libmonitoring-plugin-perl"
    service="nagios-nrpe-server-args"
    cat <<EOF >/tmp/samba-common.seed
samba-common    samba-common/dhcp       boolean false
EOF
    /usr/bin/apt-get install -q -y debconf-utils
    /usr/bin/debconf-set-selections /tmp/samba-common.seed
    /bin/rm -f /tmp/samba-common.seed
    prepare_debian_like
    if [ ${mydistro} == "debian10" ] || [ ${mydistro} == "debian11" ]; then
        dpkg -i ${script_path}linux/debian/nagios-nrpe-server-args_3.2.1-2.1_amd64.deb
    elif [ ${mydistro} == "debian12" ]; then
        dpkg -i ${script_path}linux/debian/nagios-nrpe-server-args_4.1.0-1_amd64.deb
    fi
    setup_service
    fix_plugins
    enable_systemd_service
    ;;
sles15* | opensuse-leap15*)
    packages="nrpe perl-Nagios-Plugin bc sysstat perl-File-Slurp perl-Readonly"
    service="nrpe"
    config_path="/etc/nrpe.d"
    prepare_suse_like
    setup_service
    fix_service
    enable_chkconfig_service
    ;;
*)
    echo "****************************************"
    echo " This Operating System is NOT SUPPORTED "
    echo "****************************************"
    ;;
esac

echo ""
echo "======================"
echo "INSTALLATION COMPLETED"
echo "======================"

trap - EXIT
