#!/usr/bin/perl -w

# check_unix_disk_usage Nagios Plugin
#
# TComm - Carlos Peris Pla
#
# This nagios plugin is free software, and comes with ABSOLUTELY 
# NO WARRANTY. It may be used, redistributed and/or modified under 
# the terms of the GNU General Public Licence (see 
# http://www.fsf.org/licensing/licenses/gpl.txt).


# MODULE DECLARATION

use strict;
use Nagios::Plugin;


# FUNCTION DECLARATION

sub CreateNagiosManager ();
sub CheckArguments ();
sub PerformCheck ();
sub AdjustUnit ();


# CONSTANT DEFINITION

use constant NAME => 	'check_Disk usage';
use constant VERSION => '0.1b';
use constant USAGE => 	"Usage:\ncheck_unix_disk_usage -w <disk_threshold%> -c <disk_threshold%>\n".
    "\t\t[-V <version>]\n";
use constant BLURB => 	"This plugin checks the disk space usage in all filesystems of a UNIX system and compares the percentage\n".
    "of disk space usage in any of the filesystem with warning and critical thresholds.\n";
use constant LICENSE => "This nagios plugin is free software, and comes with ABSOLUTELY\n".
    "no WARRANTY. It may be used, redistributed and/or modified under\n".
    "the terms of the GNU General Public Licence\n".
    "(see http://www.fsf.org/licensing/licenses/gpl.txt).\n";
use constant EXAMPLE => "\n\n".
    "Examples:\n".
    "\n".
    "check_unix_disk_usage.pl -w 80 -c 90\n".
    "\n".
    "It returns WARNING if disk space usage is higher than 80% in any of the filesystems in system, and CRITICAL if the disk\n".
    "usage is higher than 90% in any of the filesystems in system.\n".
    "In other cases  it returns OK if check has been performed succesfully.\n\n".
    "Note: thresholds of both WARNING and CRITICAL must be passed in percentage.\n";
use constant FILESYSTEMS_EXCLUDED => '-x tmpfs -x devtmpfs -x ramfs -x debugfs -x devpts -x iso9660 -x nfs -x nfs4 -x proc -x romfs -x smbfs -x squashfs';

								
# VARIABLE DEFINITION

my $Nagios;
my $Error;
my $PluginResult;
my $PluginOutput;


# MAIN FUNCTION

# Get command line arguments
$Nagios = &CreateNagiosManager(USAGE, VERSION, BLURB, LICENSE, NAME, EXAMPLE);
eval {$Nagios->getopts};

if (!$@) {
	# Command line parsed
	if (&CheckArguments($Nagios, \$Error)) {
		# Argument checking passed
		$PluginResult = &PerformCheck($Nagios, \$PluginOutput)
	}
	else {
		# Error checking arguments
		$PluginOutput = $Error;
		$PluginResult = UNKNOWN;
	}
	$Nagios->nagios_exit($PluginResult,$PluginOutput);
}
else {
	# Error parsing command line
	$Nagios->nagios_exit(UNKNOWN,$@);
}

		
	
# FUNCTION DEFINITIONS

# Creates and configures a Nagios plugin object
# Input: strings (usage, version, blurb, license, name and example) to configure argument parsing functionality
# Return value: reference to a Nagios plugin object

sub CreateNagiosManager() {
	# Create GetOpt object
	my $Nagios = Nagios::Plugin->new(usage => $_[0], version =>  $_[1], blurb =>  $_[2], license =>  $_[3], plugin =>  $_[4], extra =>  $_[5]);					

	# Add argument warning
	$Nagios->add_arg(spec => 'warning|w=s',
				help => "Warning threshold (in percentage). Format: <disk_threshold%>",
				required => 1);
	# Add argument critical
	$Nagios->add_arg(spec => 'critical|c=s',
				help => "Critical threshold (in percentage). Format: <disk_threshold%>",
				required => 1);
								
	# Return value
	return $Nagios;
}


# Checks argument values and sets some default values
# Input: Nagios Plugin object
# Output: reference to Error description string
# Return value: True if arguments ok, false if not

sub CheckArguments() {
	my ($Nagios, $Error) = @_;
	my $warning;
	my $critical;		

	
	# Check Warning thresholds list
	$warning = $Nagios->opts->warning;
	if ($warning !~/^(@?(\d+(\.\d+)?|(\d+|~):(\d+(\.\d+)?)?))?$/){
		${$Error} = "Invalid disk Warning threshold in $warning";
		return 0;
	}

	
	# Check Critical thresholds list
	$critical = $Nagios->opts->critical;
	if ($critical !~/^(@?(\d+(\.\d+)?|(\d+|~):(\d+(\.\d+)?)?))?$/){
		${$Error} = "Invalid disk Critical threshold in $critical";
		return 0;
	}
	
	return 1;
}


# Performs whole check: 
# Input: Nagios Plugin object, reference to Plugin output string, disk Unit, Swap Unit, referece to WVRange, reference to CVRange
# Output: Plugin output string
# Return value: Plugin return value

sub PerformCheck() {
	my ($Nagios, $PluginOutput) = @_;
	my $WarningThreshold;
	my $CriticalThreshold;
	my $Command;
	my $FsCounter;
	my @DfResult;
	my @FsFields;
	my $Used;
	my $Available;
	my $MountedOn;
	my $Total;
	my $UsedPercentage;
	my $WarningInBytes;
	my $CriticalInBytes;
	my $OutputUsed;
 	my $PluginReturnValue = UNKNOWN;
 	my $FsPluginReturnValue;
 	my $OkFlag = 0;
 	my $WarningFlag = 0;
 	my $CriticalFlag = 0;
 	my $PerformanceData="";
 	
 	${$PluginOutput} = "";
 	
 	$WarningThreshold = $Nagios->opts->warning;
 	$CriticalThreshold = $Nagios->opts->critical;
 	
 	# Recovering space used in filesystems, checking plugin state and set both output and perfdata
	$Command = "df -Pl ".FILESYSTEMS_EXCLUDED."|";
	$FsCounter = 0;
	open(DF, $Command);
	while(<DF>){
		if ($FsCounter >= 1) {
			chomp();
        	next if (/^\/proc\b/);
       		my @FsFields = split;
       		
       		# Getting values
			$Used = $FsFields[2];
			$Used *= 1024; # To Bytes
			$Available = $FsFields[3];
			$Available *= 1024; # To Bytes
			$MountedOn = $FsFields[5];
			$Total = $Used + $Available;
			$UsedPercentage = ($Used * 100.0) / $Total;
			$UsedPercentage = sprintf("%.2f", $UsedPercentage);
			$WarningInBytes = ($WarningThreshold * $Total) / 100.0;
			$WarningInBytes = sprintf("%.1f", $WarningInBytes);
			$CriticalInBytes = ($CriticalThreshold * $Total) / 100.0;
			$CriticalInBytes = sprintf("%.1f", $CriticalInBytes);
			$OutputUsed = &AdjustUnit($Used);
		
			# Plugin return value and plugin output
			$FsPluginReturnValue = $Nagios->check_threshold(check => $UsedPercentage,warning => $WarningThreshold,critical => $CriticalThreshold);
			${$PluginOutput} .= "$MountedOn: $OutputUsed ($UsedPercentage%) used";
			if ($FsPluginReturnValue eq OK){
				$OkFlag = 1;
				${$PluginOutput} .= ";";
			}
			elsif ($FsPluginReturnValue eq WARNING) {
				${$PluginOutput} .= " (warning threshold is set to $WarningThreshold%);";
				$WarningFlag = 1;
			}
			elsif ($FsPluginReturnValue eq CRITICAL) {
				${$PluginOutput} .= " (critical threshold is set to $CriticalThreshold%);";
				$CriticalFlag = 1;
			}
			
			# Plugin perfdata
			$PerformanceData .= "$MountedOn=$Used"."B;$WarningInBytes;$CriticalInBytes;0;$Total ";
		}
		$FsCounter++;
	}
	close (DF);
	
	# Establishing PluginReturnValue
	if ($CriticalFlag) {
		$PluginReturnValue = CRITICAL;
	}
	elsif ($WarningFlag) {
		$PluginReturnValue = WARNING;
	}
	elsif ($OkFlag) {
		$PluginReturnValue = OK;
	}

	chop( ${$PluginOutput} );
	chop( $PerformanceData );
	# Output with performance data:
	${$PluginOutput} .= " | $PerformanceData";

 	return $PluginReturnValue;
}



# Sets the unit of disk space used by filesystem
# Input: Used (disk space used, in bytes, in a filesystem)
# Return: Output Used
sub AdjustUnit () {
	my ($Used) = @_;
	my $OutputUsed = "";
	
	# Conversion factors
	my $KiB = 1024.0;
	my $MiB = 1024.0 * 1024.0;
	my $GiB = 1024.0 * 1024.0 * 1024.0;
	my $TiB = 1024.0 * 1024.0 * 1024.0 * 1024.0;
	
	# Adjusting unit
	if ($Used < $KiB) { # In Bytes
		$OutputUsed .= "$Used"."B";
		return $OutputUsed;
	}
	elsif ($KiB <= $Used and $Used < $MiB) { # In Kibibytes
		$Used = $Used / $KiB;
		$Used = sprintf("%.2f", $Used);
		$OutputUsed .= "$Used"."KiB";
		return $OutputUsed;
	}
	elsif ($MiB <= $Used and $Used < $GiB) { # In Mebibytes
		$Used = $Used / $MiB;
		$Used = sprintf("%.2f", $Used);
		$OutputUsed .= "$Used"."MiB";
		return $OutputUsed;
	}
	elsif ($GiB <= $Used and $Used < $TiB) { # In Gibibytes
		$Used = $Used / $GiB;
		$Used = sprintf("%.2f", $Used);
		$OutputUsed .= "$Used"."GiB";
		return $OutputUsed;
	}
	else { # In Tebibytes
		$Used = $Used / $TiB;
		$Used = sprintf("%.2f", $Used);
		$OutputUsed .= "$Used"."TiB";
		return $OutputUsed;
	}
}
